//****************************************************************************
// MODULE NAME: Reports
//
// DESCRIPTION:  Reads in New Loans Report and Loan Status Report, interfaces
//   with Oracle Database and updates info.
//
//****************************************************************************
#ifndef report_hpp
#define report_hpp

#include <cstdio>
#include <iostream>
#include <fstream>
#include <string>
#include <cstdlib>
#include <iomanip>
#include <unistd.h>
#include <sys/wait.h>



class Reports {

 public:
   void new_loans_report( void );
   //*******************************************************************
   // Params: None.
   //
   // Inputs: New Loans Report File
   //
   // Outputs: writes to database
   //
   // Description: Reads the New Loans Report and updates both Loan
   //   Portfolio and Borrower Database, New Lenders must be added first
   //   in the Lender CRUD.
   //*******************************************************************
 
   void loan_status_report( void );
   //*******************************************************************
   // Params: None.
   //
   // Inputs: Loan Status Report File
   //
   // Outputs: Updates Portfolio and Borrower Database
   //
   // Description: Reads the Loan Status Report and passes new info to
   //   Loan Database, Database will then update Status and Borrower 
   //   Risk as necessary.
   //*******************************************************************
   bool verified( int );

   enum {
     DRY_RUN,   // checking entire report for errors, not fixing
     GAME_TIME  // adding or updating loans now, so fix bad records
   };
     
   enum {
     GOOD = 0,   // Good standing
     LATE = 1,   // Late standing
     DEFAULT = 2 // Default standing
   };

   enum {
     FM = 0, // Fixed Rate
     ARM = 1 // Adjustable Rate
   };

   enum {
     JUMBO = 0,  // Jumbo Loan $250000+
     REGULAR = 1 // Regular Loan $249999.99-
   };

   int Record_Type;             // 0-99
   double Loan_ID;              // Loan ID 0-9999999999
   double Lender_ID;            // Lender ID 0-99999999
   int FMCO_Purch;              // FMCO Purchase 0/1
   double Original_Value;       // Original Value 1000-500000
   double Interest;             // Expected total Interest Payed 
   float Interest_Rate;         // Interest Rate 00.001-99.999
   int Rate_Type;               // FM or ARM
   double ARM_Principal;        // Principal Remaining 1000-500000
   int MM;                      // Month 1-12
   int DD;                      // Day 1-31
   int YYYY;                    // Year no range
   int Term;                    // Term 1-30
   int Loan_Type;               // Loan Type JUMBO/REGULAR
   double Property_ID;          // Property ID 0-9999999999
   int Loan_Status;             // GOOD/LATE/DEFAULT
   double Monthly_Payment;      // Used to calc PR for FM loans, not used
				// for ARM
   double Borrower_ID[4];       // 0-9999999999
   char Borrower_Name[4][65];   // Name

 private:
   int get_int( char *token, int min, int max );
   //*****************************************************************
   // Params: token - string token
   //         min - minimum value of int
   //         max - maximum value of int
   // Inputs: None.
   // Outputs: None.
   // Return Value: int - integer value from token, -1 if outside of
   //                     range
   // Description: Parses token for an integer within the min and max 
   //   values, if outside of range -1 is returned instead.
   //****************************************************************

   double get_double( char*, double, double );

   float get_float( char *token );
   //*****************************************************************
   // Params: token - string token
   // Inputs: None.
   // Outputs: None.
   // Return Value: float - floating point value from token, -1 if
   //                        outside of range
   // Description: Parses token for a float within the min[.001] and
   //   max[99.999] values, if outside of range -1 is returned instead.
   //****************************************************************

   char* get_name( char *token );
   //****************************************************************
   // Params: token - string token
   // Inputs: None.
   // Outputs: None.
   // Return Value: char - name
   //
   // Description: Gets a valid name from the token
   //
   //****************************************************************

   void parse_line( char *buffer );
   //****************************************************************
   // Params: char *buffer - 1 line of file
   // Inputs: None.
   // Outputs: Stores all loan info from current line to data members
   // Return Value: None
   //
   // Description: Parses line from report file.
   //
   //****************************************************************

  

   void add_loan( void );

   void add_borrower( double, char* );
   void add_b_have_l( double, double );

   void update_loan( void );

   void update_b_have_l( double, double );

   bool fix_info( void );
   bool check_lender( void );

};
#endif
